      subroutine leverrier (n,a,c)

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c------------------------------------------------
c This program accompanies the book:
c          C. Pozrikidis
c Numerical Computation in Science and Engineering
c        Oxford University Press
c------------------------------------------------

c-----------------------------------------
c  Coefficients of the 
c  characteristic polynomial of an nxn matrix 
c  by the method of Leverrier
c
c  Algorithm (5.4.3)
c
c  SYMBOLS:
c  --------
c       
c  a .... square matrix
c  n .... size (rows/columns) of matrix a
c  c .... vector of polynomial coefficients
c
c  b .... iterated matrix for computation of c
c  d .... iterated matrix for computation of c
c-----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension a(10,10),c(0:10),b(10,10),d(10,10)

c---------------------------------------
c establish initial elements of matrix b 
c---------------------------------------

      Do i=1,n
        Do j=1,n
          b(i,j)=0.0D0
        End Do
        b(i,i)=1.0D0
      End Do  

c-----------
c initialize
c-----------

      c(0) = 1.0D0

c---------------------------------------
c outer loop for polynomial coefficients
c---------------------------------------

      Do 97 i=1,n             !  begin outer loop
   
        c(i)=0.0D0

c---
c update the matrix d
c---

        Do j=1,n
         Do k=1,n
          d(j,k)=0.0D0
           Do l=1,n
            d(j,k)=d(j,k)+a(j,l)*b(l,k)
           End Do
         End Do
        End Do

c---
c compute c(i)
c---
 
        tr=0.0D0
 
        Do j=1,n
         tr=tr+d(j,j)
        End Do

        c(i)=-tr/i

c---
c update the matrix b
c---

        Do j=1,n
          Do k=1,n
           b(j,k)=d(j,k)
          End Do
          b(j,j)=b(j,j)+c(i)
        End Do

  97  Continue               !  end of outer loop

c----------------------------------------------
c update coefficients for leading factor (-1)^n
c----------------------------------------------

      fact = (-1.0D0)**n

      Do i=0,n
       c(i) = fact*c(i)
      End Do

c-----
c Done
c-----

      Return
      End
